==> centraal_client_flow-0.1.12-py3-none-any.whl: centraal_client_flow/rules/integration/processor.py <==
"""Reglas de integración."""

import json
import logging
from typing import Optional

from azure.functions import Blueprint, ServiceBusMessage
from pydantic import ValidationError

from centraal_client_flow.models.schemas import (
    EntradaEsquemaUnificado,
    AuditoriaEntryIntegracion,
    IDModel,
)
from centraal_client_flow.rules.integration.strategy import (
    IntegrationStrategy,
    StrategyResult,
)
from centraal_client_flow.connections.cosmosdb import CosmosDBSingleton


class FunctionBuilder:
    """
    Clase para construir y registrar funciones de Azure basadas en la configuración proporcionada.

    Esta clase permite construir dinámicamente funciones de Azure que se desencadenan
    por mensajes de un topic de Service Bus, utilizando una estrategia de integración específica
    y un modelo de esquema de entrada unificado.
    """

    def __init__(
        self,
        function_name: str,
        topic_name: str,
        connection_str: str,
        subscription_name: str,
        integration_strategy: IntegrationStrategy,
        model_unficado: type[EntradaEsquemaUnificado],
    ):
        """
        Inicializa un constructor de funciones con los parámetros especificados.

        Args:
            function_name: Nombre único de la función que se va a registrar.
            topic_name: Nombre del topic de Service Bus desde donde se recibirán los mensajes.
            connection_str: Cadena de conexión para el Service Bus.
            subscription_name: Nombre de la suscripción en el topic de Service Bus.
            integration_strategy: Estrategia de integración que se aplicará al procesar el mensaje.
            model_unficado: Modelo de esquema unificado para validar y mapear los mensajes recibidos.
        """
        self.function_name = function_name
        self.topic_name = topic_name
        self.connection_str = connection_str
        self.subscription_name = subscription_name
        self.integration_strategy = integration_strategy
        self.model_unficado = model_unficado

    def build_function(self):
        """
        Construye la función de integración de Azure que procesa mensajes del topic de Service Bus.

        Returns:
            Una función que procesa mensajes, valida el esquema de entrada, aplica la estrategia de integración
            y realiza la integración.
        """

        def integrate_function(message: ServiceBusMessage):
            message_body = json.loads(message.get_body().decode("utf-8"))
            message_esquema = self.model_unficado.model_validate(message_body)
            output_model = self.integration_strategy.modelo_unificado_mapping(
                message_esquema
            )
            self.integration_strategy.integrate(output_model)

        return integrate_function

    def register_function(self, bp: Blueprint):
        """
        Registra la función construida en el Blueprint proporcionado.

        Args:
            bp: Blueprint de Azure Functions donde se registrará la función.

        Returns:
            El Blueprint actualizado con la función registrada.
        """
        integrate_function = self.build_function()

        bp.function_name(name=self.function_name)(
            bp.service_bus_topic_trigger(
                arg_name="message",
                topic_name=self.topic_name,
                connection=self.connection_str,
                subscription_name=self.subscription_name,
            )(integrate_function)
        )
        return bp


class IntegrationRule:
    """
    Clase para definir y registrar reglas de integración basadas en topics de Service Bus.

    Esta clase encapsula la lógica para definir una regla de integración utilizando un topic,
    una suscripción y una estrategia de integración específica.
    """

    def __init__(
        self,
        topic_name: str,
        connection_str: str,
        subscription_name: str,
        integration_strategy: IntegrationStrategy,
        model_unficado: type[EntradaEsquemaUnificado],
    ):
        """
        Inicializa una regla de integración con los parámetros especificados.

        Args:
            topic_name: Nombre del topic de Service Bus que se utilizará para la integración.
            connection_str: Cadena de conexión para el Service Bus.
            subscription_name: Nombre de la suscripción en el topic de Service Bus.
            integration_strategy: Estrategia de integración a aplicar en los mensajes procesados.
            model_unficado: Modelo de esquema unificado para validar y mapear los mensajes recibidos.
        """
        if integration_strategy.name is not None:
            self.function_name = (
                f"{integration_strategy.name.lower()}_{topic_name}_intrule"
            )
        else:
            self.function_name = f"{topic_name}_intrule"

        self.topic_name = topic_name
        self.connection_str = connection_str
        self.subscription_name = subscription_name
        self.integration_strategy = integration_strategy
        self.model_unficado = model_unficado
        self.id_esquema: Optional[IDModel] = None

    def run(self, message: dict, logger: logging.Logger) -> Optional[StrategyResult]:
        """Ejecutra la regla de integración."""
        try:
            message_esquema = self.model_unficado.model_validate(message)
            self.id_esquema = message_esquema.id
            output_model = self.integration_strategy.modelo_unificado_mapping(
                message_esquema
            )
        except ValidationError as e:
            logger.error(
                "Error antes de integración en validación %s", e.errors(), exc_info=True
            )
            return StrategyResult(
                success=False,
                response={"error_validacion": str(e.errors())},
                bodysent={"error_Validacion": True},
            )

        return self.integration_strategy.integrate(output_model)

    def register_log(
        self,
        result: StrategyResult,
        cosmos_client: CosmosDBSingleton,
        container_name: str,
    ):
        container = cosmos_client.get_container_client(container_name)
        if self.id_esquema is not None:
            entry = AuditoriaEntryIntegracion(
                id=self.id_esquema,
                regla=self.function_name,
                contenido=result.bodysent,
                sucess=result.success,
                response=result.response,
            )
            item_written = container.upsert_item(
                entry.model_dump(mode="json", exclude_none=True),
            )
            return item_written

        raise ValueError("No es posible usar registro del log.")

    def register_function(self, bp: Blueprint):
        """
        Crea y registra una función de integración en el Blueprint proporcionado.
        TODO: esto debe ser validado si es posible, actualmente no funciona!!

        Args:
            bp: Blueprint de Azure Functions donde se registrará la función de integración.

        Returns:
            El Blueprint actualizado con la función de integración registrada.
        """
        builder = FunctionBuilder(
            function_name=self.function_name,
            topic_name=self.topic_name,
            connection_str=self.connection_str,
            subscription_name=self.subscription_name,
            integration_strategy=self.integration_strategy,
            model_unficado=self.model_unficado,
        )
        return builder.register_function(bp)

==> centraal_client_flow-0.1.16-py3-none-any.whl: centraal_client_flow/rules/integration/processor.py <==
"""Reglas de integración."""

import json
import logging
from typing import Optional

from azure.functions import ServiceBusMessage
from pydantic import ValidationError

from centraal_client_flow.models.schemas import (
    EntradaEsquemaUnificado,
    AuditoriaEntryIntegracion,
    IDModel,
)
from centraal_client_flow.rules.integration.strategy import (
    IntegrationStrategy,
    StrategyResult,
)
from centraal_client_flow.connections.cosmosdb import CosmosDBSingleton
from centraal_client_flow.helpers.pydantic import serialize_validation_errors


class IntegrationRule:
    """
    Clase para definir y registrar reglas de integración basadas en topics de Service Bus.

    Esta clase encapsula la lógica para definir una regla de integración utilizando un topic,
    una suscripción y una estrategia de integración específica.
    """

    def __init__(
        self,
        topic_name: str,
        connection_str: str,
        subscription_name: str,
        integration_strategy: IntegrationStrategy,
        model_unficado: type[EntradaEsquemaUnificado],
    ):
        """
        Inicializa una regla de integración con los parámetros especificados.

        Args:
            topic_name: Nombre del topic de Service Bus que se utilizará para la integración.
            connection_str: Cadena de conexión para el Service Bus.
            subscription_name: Nombre de la suscripción en el topic de Service Bus.
            integration_strategy: Estrategia de integración a aplicar en los mensajes procesados.
            model_unficado: Modelo de esquema unificado para validar y mapear los mensajes recibidos.
        """
        if integration_strategy.name is not None:
            self.function_name = (
                f"{integration_strategy.name.lower()}_{topic_name}_intrule"
            )
        else:
            self.function_name = f"{topic_name}_intrule"

        self.topic_name = topic_name
        self.connection_str = connection_str
        self.subscription_name = subscription_name
        self.integration_strategy = integration_strategy
        self.model_unficado = model_unficado
        self.id_esquema: Optional[IDModel] = None

    def run(
        self, message: ServiceBusMessage | dict, logger: logging.Logger
    ) -> Optional[StrategyResult]:
        """Ejecuta la regla de integración."""
        if isinstance(message, ServiceBusMessage):
            message = json.loads(message.message)

        try:
            message_esquema = self.model_unficado.model_validate(message)
            self.id_esquema = message_esquema.id
            output_model = self.integration_strategy.modelo_unificado_mapping(
                message_esquema
            )
        except ValidationError as e:

            error_val_cosmos_friendly = serialize_validation_errors(e.errors())

            logger.error(
                "Error antes de integración en validación %s",
                error_val_cosmos_friendly,
                exc_info=True,
            )
            return StrategyResult(
                success=False,
                response={"error_validacion": error_val_cosmos_friendly},
                bodysent={"error_validacion": True},
            )

        return self.integration_strategy.integrate(output_model)

    def register_log(
        self,
        result: StrategyResult,
        cosmos_client: CosmosDBSingleton,
        container_name: str,
    ):
        container = cosmos_client.get_container_client(container_name)
        if self.id_esquema is not None:
            entry = AuditoriaEntryIntegracion(
                id=self.id_esquema,
                regla=self.function_name,
                contenido=result.bodysent,
                sucess=result.success,
                response=result.response,
            )
            item_written = container.upsert_item(
                entry.model_dump(mode="json", exclude_none=True),
            )
            return item_written

        raise ValueError("No es posible usar registro del log.")
